/*
** Copyright (C) 2001,2002 Sacha Faust <sacha@severus.org>
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/
package org.ideahamster.metis.util;

import java.io.*;
import java.util.*;
import faust.sacha.web.util.*;
import java.net.*;
import faust.sacha.web.bot.spider.data.*;



/** This class handles the options for Metis.
 * @author Sacha Faust
 * @version <PRE>1.4</PRE>
 */
public final class MetisOpt {

    private static int m_reportType;
    private static String m_fileListFileName;
    private static Vector m_siteList;
    
    public MetisOpt() {
        m_reportType = MetisGlobal.REPORT_TYPE_HTML;
        m_siteList = new Vector(0);
        
        // Make sure the globals are init properly
        WebGlobal.setUserAgent( MetisGlobal.METIS_USER_AGENT );
        WebGlobal.initProperties();
        
    }
//------------------------------------------------------------------------------
    /**
     * @param requestMethod The request method type. See <I>faust.sacha.web.uti.WebGlobal</I> for a listing
     * of available request method types.
     */    
    public void setRequestMethod( int requestMethod ){
        WebGlobal.setRequestMethod( requestMethod );
    }
//------------------------------------------------------------------------------
    /**
     * @param debug True will enable debugging.
     * False will disable debugging.
     */    
    public void setDebug( boolean debug ){
        WebGlobal.setDebug( debug );
        MetisGlobal.setDebug( debug );
    }
//------------------------------------------------------------------------------
    /**
     * @param proxyHost The host used for proxying.
     */    
    public void setProxyHost( String proxyHost ){
        WebGlobal.PROXY_HOST = proxyHost;
    }
//------------------------------------------------------------------------------
    /**
     * @param proxyPort The port used by the proxy server.
     */    
    public void setProxyPort( int proxyPort ){
        WebGlobal.PROXY_PORT = proxyPort;
    }
//------------------------------------------------------------------------------    
    /**
     * @param type Report type to generate. See <I>org.ideahamster.util.MetisGlobal</I> for a listing of valid
     * report types.
     */    
    public void setReportType( int type ){
        m_reportType = type;
    }
//------------------------------------------------------------------------------
    /**
     * @return The report type used.
     */    
    public static int getReportType(){
        return m_reportType;
    }
//------------------------------------------------------------------------------
    /**
     * @param username The user name used to authenticate to the server.
     */    
    public void setRequestUserName( String username ){
        WebGlobal.HTTP_USER = username;
    }
//------------------------------------------------------------------------------    
    /**
     * @param password The password used to authenticate to the server.
     */    
    public void setRequestPassword( String password ){
        WebGlobal.HTTP_PASSWORD = password;
    }
//------------------------------------------------------------------------------
    public void setAndInitSiteFileList( String fileName ){
        RandomAccessFile listFile = null;
        String tmpInput = null;
        int lineNumber = 1;
        int pos = -1;
        String siteStr = null;
        String emailDomainStr = null;
        
        try{
            listFile = new RandomAccessFile(fileName, "r");
        }
        catch( FileNotFoundException notFoundEx ){
            System.err.println( "Unable to open site list file " + fileName );
                        
            usage();
        }
        
        while( true ){
            emailDomainStr = null;
            tmpInput = null;
            try{
                tmpInput = listFile.readLine();
            }
            catch( IOException ioEx ){
                System.err.println("Error reading site list file : " + ioEx);
                break;
            }
            
            if( tmpInput ==  null )
                break;
            
            // quick cleanup
            tmpInput = tmpInput.trim();
            
            // we skip comments
            if( tmpInput.startsWith("#") ){
                lineNumber++;
                continue;
            }
            
            if( tmpInput.startsWith("[") ){
                if( !tmpInput.endsWith("]") ){
                    System.out.println("Invalid synthax in site list on line : " + lineNumber + " skipping it!");
                    lineNumber++;
                    continue;
                }
                
                pos = tmpInput.lastIndexOf(",");
                if( pos <= 0 ){     // ie: [,emaildomain] or worst
                    System.out.println("Invalid synthax in site list on line : " + lineNumber + " skipping it!");
                    lineNumber++;
                    continue;                    
                }
                
                siteStr = tmpInput.substring(1, pos);       // 1 is to skip the [
                emailDomainStr = tmpInput.substring(pos, tmpInput.length() -1);   // -1 is to skip the ]
            }
            else{
                siteStr = tmpInput;
            }
            
            if( !siteStr.toLowerCase().startsWith("http:") || !siteStr.toLowerCase().startsWith("https:") ){
                // we default to http
                siteStr = "http://" + siteStr;
            }
            
            if( emailDomainStr == null )
                emailDomainStr = getEmailDomain(siteStr);
            
            if( MetisGlobal.METIS_DEBUG )
                System.err.println("MetisOpt::setAndInitSiteFileList() : adding : " + siteStr + ":" + emailDomainStr + " : to the site list");
            
            m_siteList.add( (Object)new HostAndEmailDomain(siteStr, emailDomainStr) );
            lineNumber++;
        }
        
        try{
            listFile.close();
        }
        catch( IOException ioeEx ){
        }        
    }
//------------------------------------------------------------------------------
    public void setAndInitURLList( String fileName ){
        Hashtable urlList = null;
        RandomAccessFile listFile = null;
        String tmpInput = null;
        Vector fileList = null;
        Vector folderList = null;
        int pos = 0;
        String typeStr = null;
        String urlStr = null;
        int lineNumber = 1;
        
        urlList = new Hashtable(0);
        fileList = new Vector(0);
        folderList = new Vector(0);
        
        urlList.put("file", fileList);
        urlList.put("folder", folderList);
        
        try{
            listFile = new RandomAccessFile(fileName, "r");
        }
        catch( FileNotFoundException notFoundEx ){
            System.err.println( "Unable to open url list file " + fileName );
            
            usage();
        }
        
        while( true ){
            try{
                tmpInput = listFile.readLine();
            }
            catch( IOException ioEx ){
                System.err.println("Error reading url list file : " + ioEx );
                break;
            }
            
            if( tmpInput == null )
               break;
            
            //quick cleanup
            tmpInput = tmpInput.trim();
            
            // skipping comments
            if( tmpInput.startsWith("#") ){
                lineNumber++;
                continue;
            }
            
            // defined url type
            if( tmpInput.startsWith("[") && tmpInput.endsWith("]") ){
                tmpInput = tmpInput.substring( 1, tmpInput.length() -1 );

                /*
                 *  if pos is not 4 or 6 there is an error. 
                 *  Why?
                 *      File:<---4
                 *      Folder:<--6
                 */
                pos = tmpInput.indexOf(":");
                if( (pos != 4) && (pos != 6) ){
                    if( MetisGlobal.METIS_DEBUG )
                        System.err.println("MetisOpt::setAndInitURLList() : pos is : " + pos);
                    
                    System.out.println("Invalid synthax in url list on line : " + lineNumber + " skipping it!");
                    
                    lineNumber++;
                    continue;
                }
                typeStr = tmpInput.substring(0, pos).toLowerCase();
                urlStr = tmpInput.substring( pos + 1, tmpInput.length() ).trim();
                if( typeStr.equals("file") ){
                    if( MetisGlobal.METIS_DEBUG )
                        System.err.println("MetisOpt::setAndInitURLList() : adding to fileList : " + urlStr);
                        
                    fileList.add( (Object)urlStr );
                }
                else if( typeStr.equals("folder") ){
                    if( MetisGlobal.METIS_DEBUG )
                        System.err.println("MetisOpt::setAndInitURLList() : adding to folderList : " + urlStr);
                        
                    folderList.add( (Object)urlStr );
                }
            }
            else{       // default to file
                if( MetisGlobal.METIS_DEBUG )
                    System.err.println("MetisOpt::setAndInitURLList() : default adding to fileList : " + urlStr);
                
                    fileList.add( (Object)tmpInput );
            }
            
            lineNumber++;
        }
        
        /*
         *  We only add the list if it contains 1 element or more
         */
        if( urlList.size() > 0 )
            WebGlobal.URLS_TO_INCLUDE = urlList;
        
        try{
            listFile.close();
        }
        catch( IOException ioeEx ){
        }
   
    }
//------------------------------------------------------------------------------    
    public void addSiteToList( String site, String emailDomain ){
        if( site == null )
            throw ( new NullPointerException("MetisOpt::addSiteToList : site is null") );
                
        if( !site.toLowerCase().startsWith("http:") && !site.toLowerCase().startsWith("https:") ){
            // we default to  http
            site = "http://" + site;
        }
        
        if( emailDomain == null )
            emailDomain = getEmailDomain(site);
        
        m_siteList.add( (Object)new HostAndEmailDomain(site, emailDomain) );
    }
//------------------------------------------------------------------------------    
    public static Vector getSiteList(){
        return m_siteList;
    }
//------------------------------------------------------------------------------ 
    /** This method will validate if the required and optional options are
     * all initialized correctly.
     * @return True : all options were set correctly.
     * False : one of the options was not initiated correctly.
     */    
    public boolean validateOptions(){
        
        if( m_siteList.size() <= 0 ){
            System.err.println("You need to specify a site!");
            return false;
        }
        
        switch( m_reportType ){
            case MetisGlobal.REPORT_TYPE_ASCII:
            case MetisGlobal.REPORT_TYPE_HTML:
                break;
                
            default:
                System.err.println("Unknown report type : " + m_reportType );
                return false;
        }
        
        if( (WebGlobal.PROXY_PORT <= 0) || (WebGlobal.PROXY_PORT > 6535) ){
            System.err.println("Invalid proxy port specified : " + WebGlobal.PROXY_PORT );
            return false;
        }
        
        return true;
    }
//------------------------------------------------------------------------------
    public String getEmailDomain( String host ){
        URLInfo url = null;
        DomainInfo domInfo = null;
        String emailDomain = null;
        
        try{
            url = new URLInfo( host );
        }
        catch( MalformedURLException urlEx ){
            System.err.println("MetisOpt::setEmailDomain : unable to set email domain urlEx : " + urlEx );
            return null;
        }
        
        try{
            domInfo = new DomainInfo(url.getHost());
        }
        catch( UnknownHostException hostEx ){
            System.err.println("MetisOpt::setEmailDomain : unable to set email domain hostEx : " + hostEx );
            return null;
        }
        
        emailDomain = domInfo.getDomainStruct(2);
        
        if( MetisGlobal.METIS_DEBUG )
            System.err.println("MetisOpt::setEmailDomain : " + emailDomain);
        
        return emailDomain;
    }
//------------------------------------------------------------------------------
    /** Prints the usage for Metis.
     */    
    public static void usage(){

        System.out.println(MetisGlobal.METIS_USER_AGENT + " by " + MetisGlobal.METIS_AUTHOR + " : " + MetisGlobal.METIS_AUTHOR_EMAIL );
        System.out.println("Usage : ");
        
        System.out.println("\t-h [site name]");
        System.out.println("\t\tExample of valid site name:");
        System.out.println("\t\t\thttp://site.com");
        System.out.println("\t\t\thttps://site.com");
        System.out.println("\t\t\thttps://site.com:port");        
        System.out.println("\t\t\tsite.com");
        System.out.println("\t\t\thttp://site.com/startfolder");
        
        System.out.println("\t-D : enables debug mode");
        //System.out.println("\t-R : allow redirect.");
        //System.out.println("\t-S : allow strict redirect. Will only follow redirect to same domain");

        System.out.println("\t-U [site username]");
        System.out.println("\t-P [site password]");
        
        System.out.println("\t-X [proxy host]");
        System.out.println("\t-C [proxy port]");
        
        System.out.println("\t-T [report type] : Default is HTML");
        System.out.println("\t\treport types:");
        System.out.println("\t\t\t" + MetisGlobal.REPORT_TYPE_ASCII + " : ASCII");
        System.out.println("\t\t\t" + MetisGlobal.REPORT_TYPE_HTML + " : HTML");
        
        System.out.println("\t-l [site list file]");
        
        System.out.println("\t-F [urls to include file]");
        
        System.out.println("\t-E [email domain for host specified by -h]");
        
        System.exit(1);
    }
//------------------------------------------------------------------------------   
}
